@@ -1,5 +1,6 @@
 from configparser import SafeConfigParser
 import copy
+from datetime import datetime
 import gettext
 import json
 import locale
@@ -13,14 +14,19 @@ except ImportError:  # pragma: no coverage
 
 from ._version import __version__
 from crtools import gdoc
+from crtools.models import Demerit, MemberVacation
 
 logger = logging.getLogger(__name__)
 
 PYPI_URL = 'https://pypi.org/pypi/crtools/json'
 LOCALE_LIST = {
+    'cn': 'Chinese',
+    'de': 'German',
     'en': 'English (default)',
     'fr': 'French',
-    'cn': 'Chinese'
+    'it': 'Italian',
+    'pt': 'Portuguese',
+    'ru': 'Russian'
 }
 
 LOCALE_NOT_FOUND_ERROR_TEMPLATE = 'Locale "{}" not defined. Please use one of the following:\n'
@@ -48,7 +54,8 @@ config_defaults = {
     },
     'activity': {
         'threshold_warn'                : 7,
-        'threshold_kick'                : 21
+        'threshold_kick'                : 21,
+        'min_days_to_promote'           : 0
     },
     'score' : {
         'min_clan_size' :               46,
@@ -68,27 +75,39 @@ config_defaults = {
         'threshold_promote' :           160,
         'threshold_demote' :            0,
         'threshold_kick' :              0,
-        'threshold_warn' :              30
+        'threshold_warn' :              30,
+        'new_member_grace_period_days': 3,
+        'war_win_rate_leaderboard_min_battles': 3
+    },
+    'history': {
+        'num_wars_to_track':            10
     },
     'members': {
         'blacklist'                     : [],
         'no_promote'                    : [],
+        'kicked'                        : [],
+        'warned'                        : [],
         'vacation'                      : [],
-        'safe'                          : []
+        'safe'                          : [],
+        'custom'                        : {}
     },
     'member_table': {
         'show_rank'                     : True,
         'show_rank_previous'            : False,
         'show_name'                     : True,
+        'show_xp_level'                 : False,
         'show_score'                    : True,
         'show_trophies'                 : True,
         'show_donations'                : True,
         'show_donations_received'       : False,
+        'show_time_in_clan'             : False,
         'show_last_seen'                : False,
         'show_days_inactive'            : True,
         'show_war_win_rate'             : False,
         'show_current_war'              : True,
-        'show_warlog'                   : True
+        'show_warlog'                   : True,
+        'show_war_readiness'            : False,
+        'calc_war_readiness'            : False
     },
     'discord' : {
         'webhook_default'                   : '',
@@ -109,6 +128,7 @@ config_defaults = {
     },
     'crtools' : {
         'debug'                         : False,
+        'timestamp'                     : datetime.utcnow(),
         'locale'                        : 'en',
         'version'                       : __version__,
         'latest_version'                : __version__,
@@ -118,12 +138,15 @@ config_defaults = {
 
 def __localize_strings(locale_id):
 
-    try:
-        locale.setlocale(locale.LC_TIME, (locale_id, 'UTF-8'))
-    except locale.Error:
+    logger.debug('specified locale: "{}"'.format(locale_id))
+    if locale_id not in LOCALE_LIST:
         print(LOCALE_NOT_FOUND_ERROR_TEMPLATE.format(locale_id))
         exit()
 
+    try:
+        locale.setlocale(locale.LC_TIME, (locale_id, 'UTF-8'))
+    except locale.Error:
+        print('Locale time setting not found in your os for "{}"'.format(locale_id))
 
     localedir = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'locale')
     translate = gettext.translation('crtools', localedir, languages=[locale_id], fallback=True)
@@ -181,9 +204,12 @@ def __localize_strings(locale_id):
         'labelDonationsRecieved'    : _('Donations received'),
         'labelDaysInactive'         : _('Days Inactive'),
         'labelDays'                 : _('Days'),
+        'labelMonths'               : _('Months'),
+        'labelYears'                : _('Years'),
         'labelDay'                  : _('Day'),
         'labelMinutes'              : _('Minutes'),
         'labelHours'                : _('Hours'),
+        'labelTimeInClan'           : _('Time in Clan'),
         'labelLastSeen'             : _('Last Seen'),
         'labelCurrentWar'           : _('Current War'),
         'labelNotInClan'            : _('Not in clan'),
@@ -200,6 +226,11 @@ def __localize_strings(locale_id):
         'labelWarDayBattles'        : _('War day battles'),
         'labelWarDayWins'           : _('War day wins'),
         'labelWarWinRate'           : _('War win rate'),
+        'labelWarReadiness'         : _('War readiness'),
+        'labelWarWinRateDisclaimer' : _('Must complete at least %d battles of the last %d'),
+        'labelCollectionWinRate'    : _('Collection win rate'),
+        'labelCollectionCardsWon'   : _('Collection cards won'),
+        'labelWarScoreAverage'      : _('War score average'),
         'labelWarScore'             : _('War score'),
         'labelWarLeague'            : _('War league'),
         'labelWarDate'              : _('{month}/{day}'),
@@ -247,6 +278,7 @@ def __localize_strings(locale_id):
         'suggestionKick'            : _('Members with a <strong class="bad">score below 0</strong> will be recommended for kicking or demotion.'),
         'suggestionInactive'        : _('Members inactive for <strong class="bad">{days_inactive} days</strong> will be kicked.'),
         'suggestionPromote'         : _('A member who achieves <strong class="good">{points} points</strong> is elegible for promotion to <strong>Elder</strong> at the discretion of leadership.'),
+        'suggestionPromoteMinDays'  : _('Members must be in the clan for a at least <strong>{} days</strong> to be promoted.'),
         'suggestionRecruit'         : _('<strong>Recruit new members!</strong> The team needs some fresh blood.'),
         'suggestionNone'            : _('No suggestions at this time. The clan is in good order.'),
         'suggestionKickBlacklist'   : _('Kick <strong>{name}</strong>. Member is blacklisted.'),
@@ -273,6 +305,7 @@ def __localize_strings(locale_id):
         'roleMember'                : _('Member'),
         'roleBlacklisted'           : _('Blacklisted. Kick!'),
         'roleVacation'              : _('On vacation'),
+        'roleVacationUntil'         : _('Vacation until {}'),
         'roleInactive'              : _('Inactive {days} days'),
         'roleNoPromote'             : _('Never Promote'),
 
@@ -295,8 +328,9 @@ def __localize_strings(locale_id):
         'league-master-1'           : _('Master I'),
         'league-master-2'           : _('Master II'),
         'league-master-3'           : _('Master III'),
-        'league-champion'           : _('Grand Champion'),
+        'league-champion'           : _('Champion'),
         'league-grand-champion'     : _('Grand Champion'),
+        'league-royal-champion'     : _('Royal Champion'),
         'league-ultimate-champion'  : _('Ultimate Champion'),
 
         'discord-header-war-nag'    : _('**{} hours** left on {} day. Members have **not** completed all battles:'),
@@ -383,20 +417,37 @@ def __parse_value(new_value, template_value):
 def __get_version_info(config):
     logger.debug('Grabbing current version from PyPI')
 
-    req = requests.get(PYPI_URL)
-    latest_version = current_version = parse(config['crtools']['version'])
-    if req.status_code == requests.codes.ok:
-        j = json.loads(req.text)
-        releases = j.get('releases', [])
-        for release in releases:
-            ver = parse(release)
-            latest_version = max(latest_version, ver)
+    try:
+        req = requests.get(PYPI_URL)
+        latest_version = current_version = parse(config['crtools']['version'])
+        if req.status_code == requests.codes.ok:
+            j = json.loads(req.text)
+            releases = j.get('releases', [])
+            for release in releases:
+                ver = parse(release)
+                latest_version = max(latest_version, ver)
 
-    print('crtools v{}'.format(current_version))
-    if latest_version > current_version:
-        config['crtools']['latest_version'] = '{}'.format(latest_version)
-        config['crtools']['update_available'] = True
-        print('*** update available: crtools v{} ***'.format(latest_version))
+        print('crtools v{}'.format(current_version))
+        if latest_version > current_version:
+            config['crtools']['latest_version'] = '{}'.format(latest_version)
+            config['crtools']['update_available'] = True
+            print('*** update available: crtools v{} ***'.format(latest_version))
+    except:
+        logger.debug('Could not contact PyPI. Continuing as normal.')
+
+    return config
+
+def __process_special_status(config):
+    for demerit_type in ['blacklist', 'no_promote']:
+        demerits = {}
+        for tag in config['members'][demerit_type]:
+            demerits[tag] = Demerit(tag=tag, status=demerit_type)
+        config['members'][demerit_type] = demerits
+
+    vacations = {}
+    for tag in config['members']['vacation']:
+        vacations[tag] = MemberVacation(tag=tag)
+    config['members']['vacation'] = vacations
 
     return config
 
@@ -406,7 +457,7 @@ def load_config_file(config_file_name=None, check_for_update=False, locale=None)
 
     config = copy.deepcopy(config_defaults)
 
-    if os.path.isfile(config_file_name):
+    if config_file_name and os.path.isfile(config_file_name):
         parser = SafeConfigParser()
         parser.read(config_file_name)
 
@@ -420,9 +471,10 @@ def load_config_file(config_file_name=None, check_for_update=False, locale=None)
 
     config = __validate_paths(config)
     config = __validate_crtools_settings(config)
+    config = __process_special_status(config)
 
-    if config['google_docs']['api_key'] and config['google_docs']['sheet_id']:
-        config = gdoc.get_member_data_from_sheets(config)
+    # Augment from Google Sheet
+    config = gdoc.get_member_data_from_sheets(config)
 
     if check_for_update:
         config = __get_version_info(config)
